/*
 * Copyright (C) 2017 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#if ENABLE(DFG_JIT)

#include "Structure.h"

namespace JSC { namespace DFG {

class Graph;

class RegisteredStructure {
public:
    RegisteredStructure() = default;

    ALWAYS_INLINE Structure* get() const { return m_structure; }
    Structure* operator->() const { return get(); }

    bool operator==(const RegisteredStructure& other) const
    {
        return get() == other.get();
    }

    bool operator!=(const RegisteredStructure& other) const
    {
        return !(*this == other);
    }

    explicit operator bool() const
    {
        return !!get();
    }

private:
    friend class Graph;

    RegisteredStructure(Structure* structure)
        : m_structure(structure)
    {
        ASSERT(structure);
    }

    static RegisteredStructure createPrivate(Structure* structure)
    {
        return RegisteredStructure(structure);
    }

    Structure* m_structure { nullptr };
};

} } // namespace JSC::DFG

#endif // ENABLE(DFG_JIT)
