/*
 * Copyright (C) 2012, 2014, 2015 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "DFGValueSource.h"

#if ENABLE(DFG_JIT)

namespace JSC { namespace DFG {

void ValueSource::dump(PrintStream& out) const
{
    switch (kind()) {
    case SourceNotSet:
        out.print("NotSet");
        break;
    case SourceIsDead:
        out.print("IsDead");
        break;
    case ValueInJSStack:
        out.print("JS:", virtualRegister());
        break;
    case Int32InJSStack:
        out.print("Int32:", virtualRegister());
        break;
    case Int52InJSStack:
        out.print("Int52:", virtualRegister());
        break;
    case CellInJSStack:
        out.print("Cell:", virtualRegister());
        break;
    case BooleanInJSStack:
        out.print("Bool:", virtualRegister());
        break;
    case DoubleInJSStack:
        out.print("Double:", virtualRegister());
        break;
    case HaveNode:
        out.print("Node(", m_value, ")");
        break;
    default:
        RELEASE_ASSERT_NOT_REACHED();
        break;
    }
}

void ValueSource::dumpInContext(PrintStream& out, DumpContext*) const
{
    dump(out);
}

} } // namespace JSC::DFG

#endif // ENABLE(DFG_JIT)

